@@ -1,11 +1,34 @@
-if (getRversion() < "4.2.0") {
 
 
-## gettext(trim = TRUE) was added in R 4.2.0
-gettext <- evalq(envir = .BaseNamespaceEnv,
-function (..., domain = NULL, trim = TRUE)
+if (getRversion() < "3.0.0") {
+
+
+delayedAssign(".C_mapply", getNativeSymbolInfo("do_mapply", PACKAGE = "base"))
+
+
+.mapply <- function (FUN, dots, MoreArgs)
+.Call(.C_mapply, match.fun(FUN), dots, MoreArgs, environment())
+
+
+.setprseen2 <- function (ptr)
+.External2(.C_setprseen2, ptr)
+
+
+parse <- evalq(envir = .BaseNamespaceEnv,
+function (file = "", n = NULL, text = NULL, prompt = "?", keep.source = getOption("keep.source"),
+    srcfile = NULL, encoding = "unknown")
 {
-    gettext(..., domain = domain)
+    if (missing(srcfile)) {
+        if (!missing(keep.source)) {
+            opt.keep.source <- getOption("keep.source")
+            if (isTRUE(keep.source) != isTRUE(opt.keep.source)) {
+                on.exit(options(keep.source = opt.keep.source))
+                options(keep.source = keep.source)
+            }
+        }
+        parse(file = file, n = n, text = text, prompt = prompt, srcfile = , encoding = encoding)
+    }
+    else parse(file = file, n = n, text = text, prompt = prompt, srcfile = srcfile, encoding = encoding)
 }
 )
 
@@ -13,62 +36,193 @@ function (..., domain = NULL, trim = TRUE)
 }
 
 
-if (getRversion() < "4.1.0") {
+if (getRversion() < "3.1.0") {
 
 
-## bquote(splice = TRUE) was added in R 4.1.0
-bquote <- function (expr, where = parent.frame(), splice = FALSE)
+anyNA <- function (x, recursive = FALSE)
+.External2(.C_anyNA, x, recursive)
+
+
+.anyNA.dispatch <- function (x, recursive = FALSE)
+UseMethod("anyNA")
+
+
+anyNA.data.frame <- function (x, recursive = FALSE)
+.External2(.C_anyNAdataframe, x, recursive)
+
+
+anyNA.numeric_version <- function (x, recursive = FALSE)
+.External2(.C_anyNAnumericversion, x)
+
+
+anyNA.POSIXlt <- function (x, recursive = FALSE)
+anyNA(as.POSIXct.POSIXlt(x))
+
+
+.anyNA.default <- function (x, recursive = FALSE)
+.External2(.C_anyNAdefault, x, recursive)
+
+
+}
+
+
+if (getRversion() < "3.2.0") {
+
+
+isNamespaceLoaded <- function (name)
+.External2(.C_isRegisteredNamespace, name)
+
+
+dir.exists <- function (paths)
+.External2(.C_direxists, paths)
+
+
+lengths <- function (x, use.names = TRUE)
+.External2(.C_lengths, x, use.names)
+
+
+.lengths.default <- function (x, use.names = TRUE)
+.External2(.C_lengthsdefault, x, use.names)
+
+
+## file.info() did not have argument 'extra_cols' at this time
+file.mtime <- evalq(envir = .BaseNamespaceEnv,
+function (...)
+file.info(...)$mtime
+)
+
+
+file.info <- evalq(envir = .BaseNamespaceEnv,
+function (..., extra_cols = TRUE)
 {
-    if (!is.environment(where))
-        where <- as.environment(where)
-    unquote <- function(e) {
-        if (is.pairlist(e))
-            as.pairlist(lapply(e, unquote))
-        else if (is.call(e)) {
-            if (typeof(e[[1L]]) == "symbol" && e[[1L]] == ".")
-                eval(e[[2L]], where)
-            else if (splice) {
-                if (typeof(e[[1L]]) == "symbol" && e[[1L]] == "..")
-                    stop("can only splice inside a call", call. = FALSE)
-                else as.call(unquote.list(e))
+    if (extra_cols)
+        file.info(...)
+    else file.info(...)[1:6]
+}
+)
+
+
+.isdir <- evalq(envir = .BaseNamespaceEnv,
+function (...)
+file.info(...)$isdir
+)
+
+
+} else {
+
+
+.isdir <- evalq(envir = .BaseNamespaceEnv,
+function (...)
+file.info(..., extra_cols = FALSE)$isdir
+)
+
+
+}
+
+
+if (getRversion() < "3.3.0") {
+
+
+strrep <- function (x, times)
+{
+    if (!is.character(x))
+        x <- as.character(x)
+    .External2(.C_strrep, x, as.integer(times))
+}
+
+
+startsWith <- function (x, prefix)
+.External2(.C_startsWith, x, prefix)
+
+
+endsWith <- function (x, suffix)
+.External2(.C_endsWith, x, suffix)
+
+
+}
+
+
+if (getRversion() < "3.4.0") {
+
+
+.withAutoprint <- function (exprs, evaluated = FALSE, local = parent.frame(), print. = TRUE,
+    echo = TRUE, max.deparse.length = Inf, width.cutoff = max(20, getOption("width")),
+    deparseCtrl = c("keepInteger", "showAttributes", "keepNA"),
+    spaced = FALSE, skip.echo = 0, ...)
+{
+    if (!evaluated) {
+        exprs <- substitute(exprs)
+        if (is.call(exprs)) {
+            if (exprs[[1]] == quote(`{`)) {
+                exprs <- as.list(exprs)[-1]
+                if (missing(skip.echo) &&
+                    length(exprs) &&
+                    is.list(srcrefs <- attr(exprs, "srcref")))
+                {
+                    skip.echo <- srcrefs[[1L]][7L] - 1L
+                }
             }
-            else as.call(lapply(e, unquote))
-        }
-        else e
-    }
-    is.splice.macro <- function(e) is.call(e) && typeof(e[[1L]]) == "symbol" && e[[1L]] == ".."
-    unquote.list <- function(e) {
-        p <- Position(is.splice.macro, e, nomatch = NULL)
-        if (is.null(p))
-            lapply(e, unquote)
-        else {
-            n <- length(e)
-            head <- if (p == 1)
-                NULL
-            else e[1:(p - 1)]
-            tail <- if (p == n)
-                NULL
-            else e[(p + 1):n]
-            macro <- e[[p]]
-            mexp <- eval(macro[[2L]], where)
-            if (!is.vector(mexp) && !is.expression(mexp))
-                stop("can only splice vectors")
-            c(lapply(head, unquote), mexp, as.list(unquote.list(tail)))
         }
     }
-    unquote(substitute(expr))
+    if (!is.expression(exprs))
+        exprs <- as.expression(exprs)
+    conn <- textConnection(.code2character(exprs, width.cutoff = width.cutoff, deparseCtrl = deparseCtrl))
+    on.exit(close(conn))
+    source(file = conn, local = local, print.eval = print., echo = echo,
+        max.deparse.length = max.deparse.length, skip.echo = skip.echo, ...)
 }
 
 
+} else {
+
+
+.withAutoprint <- evalq(envir = .BaseNamespaceEnv,
+function (exprs, evaluated = FALSE, local = parent.frame(), print. = TRUE,
+    echo = TRUE, max.deparse.length = Inf, width.cutoff = max(20, getOption("width")),
+    deparseCtrl = c("keepInteger", "showAttributes", "keepNA"),
+    skip.echo = 0, ...)
+{
+    if (!evaluated) {
+        exprs <- substitute(exprs)
+        if (is.call(exprs)) {
+            if (exprs[[1]] == quote(`{`)) {
+                exprs <- as.list(exprs)[-1]
+                if (missing(skip.echo) &&
+                    length(exprs) &&
+                    is.list(srcrefs <- attr(exprs, "srcref")))
+                {
+                    skip.echo <- srcrefs[[1L]][7L] - 1L
+                }
+            }
+        }
+    }
+    source(exprs = exprs, local = local, print.eval = print.,
+        echo = echo, max.deparse.length = max.deparse.length,
+        width.cutoff = width.cutoff, deparseCtrl = deparseCtrl,
+        skip.echo = skip.echo, ...)
+}
+)
+
+
 }
 
 
-if (getRversion() < "4.0.0") {
+if (getRversion() < "3.5.0") {
 
 
-deparse1 <- evalq(envir = .BaseNamespaceEnv,
-function (expr, collapse = " ", width.cutoff = 500L, ...)
-paste(deparse(expr, width.cutoff, ...), collapse = collapse)
+...length <- function ()
+.External2(.C_dotslength)
+
+
+isTRUE <- evalq(envir = .BaseNamespaceEnv,
+function (x)
+is.logical(x) && length(x) == 1L && !is.na(x) && x
+)
+
+
+isFALSE <- evalq(envir = .BaseNamespaceEnv,
+function (x)
+is.logical(x) && length(x) == 1L && !is.na(x) && !x
 )
 
 
@@ -109,172 +263,85 @@ str2lang <- function (s)
 }
 
 
-if (getRversion() < "3.5.0") {
+if (getRversion() < "4.0.0") {
 
 
-...length <- function ()
-.External2(.C_dotslength)
-
-
-isTRUE <- evalq(envir = .BaseNamespaceEnv,
-function (x)
-is.logical(x) && length(x) == 1L && !is.na(x) && x
-)
-
-
-isFALSE <- evalq(envir = .BaseNamespaceEnv,
-function (x)
-is.logical(x) && length(x) == 1L && !is.na(x) && !x
+deparse1 <- evalq(envir = .BaseNamespaceEnv,
+function (expr, collapse = " ", width.cutoff = 500L, ...)
+paste(deparse(expr, width.cutoff, ...), collapse = collapse)
 )
 
 
 }
 
 
-if (getRversion() < "3.4.0") {
+if (getRversion() < "4.1.0") {
 
 
-.withAutoprint <- function (exprs, evaluated = FALSE, local = parent.frame(), print. = TRUE,
-    echo = TRUE, max.deparse.length = Inf, width.cutoff = max(20,
-        getOption("width")), deparseCtrl = c("keepInteger", "showAttributes",
-        "keepNA"), spaced = FALSE, ...)
+## bquote(splice = TRUE) was added in R 4.1.0
+bquote <- evalq(envir = .BaseNamespaceEnv,
+function (expr, where = parent.frame(), splice = FALSE)
 {
-    if (!evaluated) {
-        exprs <- substitute(exprs)
-        if (is.call(exprs)) {
-            if (exprs[[1]] == quote(`{`))
-                exprs <- as.list(exprs[-1])
+    if (!is.environment(where))
+        where <- as.environment(where)
+    unquote <- function(e) {
+        if (is.pairlist(e))
+            as.pairlist(lapply(e, unquote))
+        else if (is.call(e)) {
+            if (typeof(e[[1L]]) == "symbol" && e[[1L]] == ".")
+                eval(e[[2L]], where)
+            else if (splice) {
+                if (typeof(e[[1L]]) == "symbol" && e[[1L]] == "..")
+                    stop("can only splice inside a call", call. = FALSE)
+                else as.call(unquote.list(e))
+            }
+            else as.call(lapply(e, unquote))
+        }
+        else e
+    }
+    is.splice.macro <- function(e) is.call(e) && typeof(e[[1L]]) == "symbol" && e[[1L]] == ".."
+    unquote.list <- function(e) {
+        p <- Position(is.splice.macro, e, nomatch = NULL)
+        if (is.null(p))
+            lapply(e, unquote)
+        else {
+            n <- length(e)
+            head <- if (p == 1)
+                NULL
+            else e[1:(p - 1)]
+            tail <- if (p == n)
+                NULL
+            else e[(p + 1):n]
+            macro <- e[[p]]
+            mexp <- eval(macro[[2L]], where)
+            if (!is.vector(mexp) && !is.expression(mexp))
+                stop("can only splice vectors")
+            c(lapply(head, unquote), mexp, as.list(unquote.list(tail)))
         }
     }
-    if (!is.expression(exprs))
-        exprs <- as.expression(exprs)
-    conn <- textConnection(.code2character(exprs, width.cutoff = width.cutoff, deparseCtrl = deparseCtrl))
-    on.exit(close(conn))
-    source(file = conn, local = local, print.eval = print., echo = echo,
-        max.deparse.length = max.deparse.length, ...)
+    unquote(substitute(expr))
 }
-
-
-} else {
-
-
-.withAutoprint <- withAutoprint
+)
 
 
 }
 
 
-if (getRversion() < "3.3.0") {
+if (getRversion() < "4.2.0") {
 
 
-strrep <- function (x, times)
+## gettext(trim = TRUE) was added in R 4.2.0
+gettext <- evalq(envir = .BaseNamespaceEnv,
+function (..., domain = NULL, trim = TRUE)
 {
-    if (!is.character(x))
-        x <- as.character(x)
-    .External2(.C_strrep, x, as.integer(times))
-}
-
-
-startsWith <- function (x, prefix)
-.External2(.C_startsWith, x, prefix)
-
-
-endsWith <- function (x, suffix)
-.External2(.C_endsWith, x, suffix)
-
-
-}
-
-
-if (getRversion() < "3.2.0") {
-
-
-isNamespaceLoaded <- function (name)
-.External2(.C_isRegisteredNamespace, name)
-
-
-dir.exists <- function (paths)
-.External2(.C_direxists, paths)
-
-
-lengths <- function (x, use.names = TRUE)
-.External2(.C_lengths, x, use.names)
-
-
-## file.info() did not have argument 'extra_cols' at this time
-file.mtime <- evalq(envir = .BaseNamespaceEnv,
-function (...)
-file.info(...)$mtime
-)
-
-
-file.info <- evalq(envir = .BaseNamespaceEnv,
-function (..., extra_cols = TRUE)
-{
-    if (extra_cols)
-        file.info(...)
-    else file.info(...)[1:6]
+    gettext(..., domain = domain)
 }
 )
 
 
-.isdir <- evalq(envir = .BaseNamespaceEnv,
-function (...)
-file.info(...)$isdir
-)
-
-
-} else {
-
-
-.isdir <- evalq(envir = .BaseNamespaceEnv,
-function (...)
-file.info(..., extra_cols = FALSE)$isdir
-)
-
-
-}
-
-
-if (getRversion() < "3.1.0") {
-
-
-anyNA <- function (x, recursive = FALSE)
-.External2(.C_anyNA, x, recursive)
-
-
-}
-
-
-if (getRversion() < "3.0.0") {
-
-
-delayedAssign(".C_mapply", getNativeSymbolInfo("do_mapply", PACKAGE = "base"))
-
-
-.mapply <- function (FUN, dots, MoreArgs)
-.Call(.C_mapply, match.fun(FUN), dots, MoreArgs, environment())
-
-
-.setprseen2 <- function (ptr)
-.External2(.C_setprseen2, ptr)
-
-
-parse <- evalq(envir = .BaseNamespaceEnv,
-function (file = "", n = NULL, text = NULL, prompt = "?", keep.source = getOption("keep.source"),
-    srcfile = NULL, encoding = "unknown")
-{
-    if (!missing(keep.source)) {
-        opt.keep.source <- getOption("keep.source")
-        if (isTRUE(keep.source) != isTRUE(opt.keep.source)) {
-            on.exit(options(keep.source = opt.keep.source))
-            options(keep.source = keep.source)
-        }
-    }
-    if (missing(srcfile))
-        parse(file = file, n = n, text = text, prompt = prompt, srcfile = , encoding = encoding)
-    else parse(file = file, n = n, text = text, prompt = prompt, srcfile = srcfile, encoding = encoding)
-}
+gettextf <- evalq(envir = .BaseNamespaceEnv,
+function (fmt, ..., domain = NULL, trim = TRUE)
+sprintf(gettext(fmt, domain = domain), ...)
 )
 
 
