@@ -1,44 +1,38 @@
-#ifndef THISPATHDEFN_H
-#define THISPATHDEFN_H
+#ifndef R_THISPATH_THISPATHDEFN_H
+#define R_THISPATH_THISPATHDEFN_H
 
 
 #include <R.h>
 #include <Rinternals.h>
-#include "rversiondefines.h"
-#include "translations.h"
+#include "backports.h"
 #include "defines.h"
-#include "thispathbackports.h"
+#include "ns-hooks.h"
+#include "print.h"
+#include "promises.h"
+#include "rversiondefines.h"
+#include "symbols.h"
+#include "translations.h"
 
 
-#if R_version_less_than(3, 4, 0)
-#define R_CurrentExpression R_NilValue
-#endif
+/* R */
 
 
-#if R_version_less_than(3, 0, 0)
-#define XLENGTH LENGTH
-#define xlength length
-#define R_xlen_t R_len_t
-#endif
+extern Rboolean pmatch(SEXP, SEXP, Rboolean);
 
 
-extern Rboolean R_existsVarInFrame(SEXP rho, SEXP symbol);
-#if R_version_less_than(4, 2, 0)
-#define existsInFrame(rho, symbol) (findVarInFrame((rho), (symbol)) != R_UnboundValue)
-#else
-#define existsInFrame(rho, symbol) R_existsVarInFrame((rho), (symbol))
-#endif
+extern void SET_PRCODE (SEXP x, SEXP v);
+extern void SET_PRENV  (SEXP x, SEXP v);
+extern void SET_PRSEEN (SEXP x, int  v);
+extern void SET_PRVALUE(SEXP x, SEXP v);
 
 
-#if R_version_at_least(3, 1, 0)
-LibExtern SEXP R_TrueValue;
-LibExtern SEXP R_FalseValue;
-LibExtern SEXP R_LogicalNAValue;
-#else
-#define R_TrueValue ScalarLogical(TRUE)
-#define R_FalseValue ScalarLogical(FALSE)
-#define R_LogicalNAValue ScalarLogical(NA_LOGICAL)
-#endif
+extern SEXP topenv(SEXP target, SEXP envir);
+
+
+extern void R_LockBinding(SEXP sym, SEXP env);
+
+
+/* handle R_THIS_PATH_DEFINES */
 
 
 #if R_version_at_least(3, 3, 0)
@@ -58,70 +52,7 @@ LibExtern SEXP R_LogicalNAValue;
 #endif
 
 
-#include "symbols.h"
-
-
-extern SEXP mynamespace,
-            promiseenv ;
-
-
-extern SEXP expr_commandArgs                              ,
-#if defined(R_THIS_PATH_HAVE_invisibleSymbol)
-            expr_invisible                                ,
-#endif
-            expr_parent_frame                             ,
-            expr_sys_call                                 ,
-            expr_sys_nframe                               ,
-            expr_sys_parents                              ,
-            expr_missing_file                             ,
-            expr_missing_input                            ,
-            expr_missing_ofile                            ,
-            expr_info_dollar_source_path                  ,
-            expr_delayedAssign_x                          ,
-            expr_knitr_output_dir                         ,
-            expr_testthat_source_file_uses_brio_read_lines,
-            expr__sys_path_toplevel                       ,
-            expr_getOption_topLevelEnvironment            ,
-            expr__toplevel_context_number                 ;
-
-
-extern SEXP makePROMISE(SEXP expr, SEXP env);
-extern SEXP makeEVPROMISE(SEXP expr, SEXP value);
-
-
-#if R_version_less_than(3, 1, 0)
-
-
-#if R_version_less_than(3, 0, 0)
-#define NAMEDMAX 2
-#define NO_REFERENCES(x) (NAMED(x) == 0)
-#define MAYBE_REFERENCED(x) (! NO_REFERENCES(x))
-#define MARK_NOT_MUTABLE(x) SET_NAMED(x, NAMEDMAX)
-#endif
-
-
-#define INCREMENT_NAMED(x) do {                                \
-    SEXP _x_ = (x);                                            \
-    if (NAMED(_x_) != NAMEDMAX)                                \
-        SET_NAMED(_x_, NAMED(_x_) + 1);                        \
-} while (0)
-
-
-#endif
-
-
-#if R_version_less_than(3, 5, 0)
-#define ENSURE_NAMEDMAX(_x_) SET_NAMED((_x_), NAMEDMAX)
-#else
-extern void (ENSURE_NAMEDMAX)(SEXP x);
-#endif
-
-
-void INCREMENT_NAMED_defineVar(SEXP symbol, SEXP value, SEXP rho);
-void MARK_NOT_MUTABLE_defineVar(SEXP symbol, SEXP value, SEXP rho);
-
-
-#if !defined(R_THIS_PATH_HAVE_invisibleSymbol)
+#if defined(R_THIS_PATH_DEFINES) && R_version_at_least(3, 0, 0)
 /* R_Visible is not part of the R API. DO NOT USE OR MODIFY IT unless you are
    absolutely certain it is what you wish to do. it is subject to change
    without notice nor back-compatibility. only the development version of this
@@ -133,76 +64,38 @@ extern Rboolean R_Visible;
 #endif
 
 
-extern SEXP R_shallow_duplicate_attr(SEXP x);
-extern SEXP installTrChar(SEXP x);
 
 
-#define streql(str1, str2) (strcmp((str1), (str2)) == 0)
 
-
-#define findFunction(symbol, rho) findFunction3(symbol, rho, R_CurrentExpression)
-extern SEXP findFunction3(SEXP symbol, SEXP rho, SEXP call);
-extern Rboolean pmatch(SEXP, SEXP, Rboolean);
-
-
-extern void SET_PRCODE (SEXP x, SEXP v);
-extern void SET_PRENV  (SEXP x, SEXP v);
-extern void SET_PRSEEN (SEXP x, int  v);
-extern void SET_PRVALUE(SEXP x, SEXP v);
-
-
-// extern int IS_BYTES(SEXP x);
-#define IS_BYTES(x) (getCharCE((x)) == CE_BYTES)
-// extern int IS_LATIN1(SEXP x);
-extern int IS_ASCII(SEXP x);
-
-
-extern void R_removeVarFromFrame(SEXP name, SEXP env);
-extern void removeFromFrame(SEXP *names, SEXP env);
+extern Rboolean needQuote(SEXP x);
 
 
 extern void UNIMPLEMENTED_TYPEt(const char *s, SEXPTYPE t);
 extern void UNIMPLEMENTED_TYPE(const char *s, SEXP x);
 
 
-#if R_version_less_than(3, 1, 0)
-extern SEXP lazy_duplicate(SEXP s);
-extern SEXP shallow_duplicate(SEXP s);
-#endif
-
-
-#if R_version_at_least(4, 1, 0)
-extern int IS_UTF8(SEXP x);
-#else
-#define IS_UTF8(x) (getCharCE((x)) == CE_UTF8)
-#endif
-
-
-extern const char *EncodeChar(SEXP x);
-
-
-extern void R_LockBinding(SEXP sym, SEXP env);
+extern const char *EncodeChar(SEXP);
 
 
 extern SEXP getInFrame(SEXP sym, SEXP env, int unbound_ok);
 #define getFromBase(sym) (getInFrame((sym), R_BaseEnv, FALSE))
 #define getFromMyNS(sym) (getInFrame((sym), mynamespace, FALSE))
+extern SEXP getInList(SEXP sym, SEXP list, int C_NULL_ok);
+
+
+void INCREMENT_NAMED_defineVar(SEXP symbol, SEXP value, SEXP rho);
+void MARK_NOT_MUTABLE_defineVar(SEXP symbol, SEXP value, SEXP rho);
+
+
+extern SEXP findFunction3(SEXP symbol, SEXP rho, SEXP call);
+extern SEXP findFunction(SEXP symbol, SEXP rho);
 
 
 extern SEXP as_environment_char(const char *what);
 
 
-#if defined(R_CONNECTIONS_VERSION_1)
-extern SEXP summaryconnection(Rconnection Rcon);
-#else
-extern SEXP summaryconnection(SEXP sConn);
-#endif
-
-
 extern SEXP errorCondition (const char *msg, SEXP call, const char **cls, int numFields);
 extern SEXP errorCondition1(const char *msg, SEXP call, const char *cls, int numFields);
-
-
 extern SEXP simpleError(const char *msg, SEXP call);
 
 
@@ -217,8 +110,10 @@ extern SEXP simpleError(const char *msg, SEXP call);
 
 
 #if defined(R_CONNECTIONS_VERSION_1)
+extern SEXP summaryconnection(Rconnection Rcon);
 extern SEXP thisPathUnrecognizedConnectionClassError(SEXP call, Rconnection Rcon);
 #else
+extern SEXP summaryconnection(SEXP sConn);
 extern SEXP thisPathUnrecognizedConnectionClassError(SEXP call, SEXP summary);
 #endif
 extern SEXP thisPathUnrecognizedMannerError         (SEXP call);
@@ -231,53 +126,90 @@ extern SEXP thisPathInAQUAError                     (SEXP call);
 extern void stop(SEXP cond);
 
 
-#ifdef _WIN32
-#define isclipboard(url) (                                             \
-                              strcmp ((url), "clipboard"     ) == 0 || \
-                              strncmp((url), "clipboard-", 10) == 0    \
-                         )
-#define mustnotbeclipboardmessage "must not be \"clipboard\" nor start with \"clipboard-\""
-#else
-#define isclipboard(url) (                                           \
-                              strcmp((url), "clipboard"    ) == 0 || \
-                              strcmp((url), "X11_primary"  ) == 0 || \
-                              strcmp((url), "X11_secondary") == 0 || \
-                              strcmp((url), "X11_clipboard") == 0    \
-                         )
-#define mustnotbeclipboardmessage "must not be \"clipboard\", \"X11_primary\", \"X11_secondary\", nor \"X11_clipboard\""
-#endif
+extern SEXP DocumentContext(void);
+extern int ofile_is_NULL(SEXP documentcontext);
 
 
-extern void assign_done(SEXP frame);
-extern void assign_default(SEXP file, SEXP frame, Rboolean check_not_directory);
-extern void assign_null(SEXP frame);
-extern void assign_chdir(SEXP file, SEXP owd, SEXP frame);
-extern void assign_file_uri(SEXP ofile, SEXP file, SEXP frame, Rboolean check_not_directory);
-extern void assign_file_uri2(SEXP description, SEXP frame, Rboolean check_not_directory);
-extern void assign_url(SEXP ofile, SEXP file, SEXP frame);
-extern void overwrite_ofile(SEXP ofilearg, SEXP frame);
+extern void assign_default(SEXP file, SEXP documentcontext, Rboolean check_not_directory);
+extern void assign_null(SEXP documentcontext);
+extern void assign_chdir(SEXP file, SEXP owd, SEXP documentcontext);
+extern void assign_file_uri(SEXP ofile, SEXP file, SEXP documentcontext, Rboolean check_not_directory);
+extern void assign_file_uri2(SEXP description, SEXP documentcontext, Rboolean check_not_directory);
+extern void assign_url(SEXP ofile, SEXP file, SEXP documentcontext);
+extern void overwrite_ofile(SEXP ofilearg, SEXP documentcontext);
 
 
-#define isurl(url) (                                           \
-                        strncmp((url), "http://" , 7) == 0 ||  \
-                        strncmp((url), "https://", 8) == 0 ||  \
-                        strncmp((url), "ftp://"  , 6) == 0 ||  \
-                        strncmp((url), "ftps://" , 7) == 0     \
-                   )
+extern SEXP sys_call(SEXP which, SEXP rho);
+#define getCurrentCall(rho) eval(expr_sys_call, (rho))
+extern int sys_parent(int n, SEXP rho);
 
 
-#if defined(R_CONNECTIONS_VERSION_1)
-#define get_connection_description(Rcon) mkCharCE((Rcon)->description, ((Rcon)->enc == CE_UTF8) ? CE_UTF8 : CE_NATIVE)
-#define get_connection_class(Rcon) ((Rcon)->class)
-#else
-#define get_connection_description(summary) STRING_ELT(VECTOR_ELT((summary), 0), 0)
-#define get_connection_class(summary) CHAR(STRING_ELT(VECTOR_ELT((summary), 1), 0))
-#endif
+extern int gui_rstudio;
+extern Rboolean has_tools_rstudio;
+extern Rboolean init_tools_rstudio(Rboolean skipCheck);
+#define in_rstudio                                             \
+    ((gui_rstudio != -1) ? (gui_rstudio) : (gui_rstudio = asLogical(getFromMyNS(_gui_rstudioSymbol))))
+#define get_debugSource                                        \
+    ((has_tools_rstudio) ? getFromMyNS(_debugSourceSymbol) : R_UnboundValue)
+
+
+extern int maybe_unembedded_shell;
+#define is_maybe_unembedded_shell                              \
+    ((maybe_unembedded_shell != -1) ? (maybe_unembedded_shell) : (maybe_unembedded_shell = asLogical(getFromMyNS(_maybe_unembedded_shellSymbol))))
+
+
+#define streql(str1, str2) (strcmp((str1), (str2)) == 0)
+
+
+// extern int IS_BYTES(SEXP x);
+#define IS_BYTES(x) (getCharCE((x)) == CE_BYTES)
+// extern int IS_LATIN1(SEXP x);
+
+
+extern int is_clipboard(const char *url);
+extern const char *must_not_be_clipboard_message;
+extern int is_url(const char *url);
+extern int is_file_uri(const char *url);
+
+
+/* doesn't work in general, for example sys.function() duplicates its return value */
+// #define identical(x, y) ((x) == (y))
+
+
+/* num.eq = TRUE                 num_as_bits = FALSE      0
+   single.NA = TRUE              NA_as_bits = FALSE       0
+   attrib.as.set = TRUE          attr_by_order = FALSE    0
+   ignore.bytecode = TRUE        use_bytecode = FALSE     0
+   ignore.environment = FALSE    use_cloenv = TRUE        16
+   ignore.srcref = TRUE          use_srcref = FALSE       0
+   extptr.as.ref = FALSE         extptr_as_ref = FALSE    0   */
+#define identical_default(x, y) ( R_compute_identical((x), (y), 16) )
+
+
+/* num.eq = FALSE                num_as_bits = TRUE       1
+   single.NA = FALSE             NA_as_bits = TRUE        2
+   attrib.as.set = FALSE         attr_by_order = TRUE     4
+   ignore.bytecode = TRUE        use_bytecode = FALSE     0
+   ignore.environment = TRUE     use_cloenv = FALSE       0
+   ignore.srcref = FALSE         use_srcref = TRUE        32
+   extptr.as.ref = TRUE          extptr_as_ref = TRUE     64  */
+#define identical_ignore_bytecode_ignore_environment(x, y) ( R_compute_identical((x), (y), 103) )
+
+
+/* num.eq = FALSE                num_as_bits = TRUE       1
+   single.NA = FALSE             NA_as_bits = TRUE        2
+   attrib.as.set = FALSE         attr_by_order = TRUE     4
+   ignore.bytecode = FALSE       use_bytecode = TRUE      8
+   ignore.environment = FALSE    use_cloenv = TRUE        16
+   ignore.srcref = FALSE         use_srcref = TRUE        32
+   extptr.as.ref = TRUE          extptr_as_ref = TRUE     64  */
+#define identical(x, y) ( R_compute_identical((x), (y), 127) )
 
 
 #if defined(R_CONNECTIONS_VERSION_1)
 typedef struct gzconn {
     Rconnection con;
+    /* there are other components to an Rgzconn, but only 'con' is needed */
 } *Rgzconn;
 #define get_description_and_class                              \
             /* as I said before, R_GetConnection() is not a part of the R API.\
@@ -288,25 +220,27 @@ typedef struct gzconn {
                 Rcon = (((Rgzconn)(Rcon->private))->con);      \
                 if (Rcon->isGzcon) error("%s; should never happen, please report!", _("invalid connection"));\
             }                                                  \
-            SEXP description = get_connection_description(Rcon);\
-            const char *klass = get_connection_class(Rcon)
-#define Rcon_or_summary(Rcon, summary) Rcon
+            SEXP description = mkCharCE(Rcon->description, (Rcon->enc == CE_UTF8) ? CE_UTF8 : CE_NATIVE);\
+            PROTECT(description); nprotect++;                  \
+            const char *klass = Rcon->class
+#define Rcon_or_summary Rcon
 #else
 #define get_description_and_class                              \
             SEXP summary = summaryconnection(ofile);           \
-            SEXP description = get_connection_description(summary);\
-            const char *klass = get_connection_class(summary); \
-            if (streql(klass, "gzcon")) error("'sys.path' not implemented for a gzcon()")
-#define Rcon_or_summary(Rcon, summary) summary
+            PROTECT(summary); nprotect++;                      \
+            SEXP description = STRING_ELT(VECTOR_ELT(summary, 0), 0);\
+            const char *klass = CHAR(STRING_ELT(VECTOR_ELT(summary, 1), 0));\
+            if (streql(klass, "gzcon")) error("'this.path' not implemented for a gzcon()")
+#define Rcon_or_summary summary
 #endif
 
 
 /* it is undesirable to have this as a #define but we also cannot
-   evaluate all the arguments. used in _syspath(), do_wrapsource(), and
-   setsyspath()
+   evaluate all the arguments. used in _syspath(), _envpath(), _srcpath(),
+   do_wrapsource(), and setpath()
  */
-#define checkfile(call, sym, ofile, frame, check_not_directory,\
-    forcepromise, assign_returnvalue,                          \
+#define checkfile(call, sym, ofile, frame, as_binding,         \
+    check_not_directory, forcepromise, assign_returnvalue,     \
     maybe_chdir, getowd, hasowd, ofilearg,                     \
     character_only, conv2utf8, allow_blank_string,             \
     allow_clipboard, allow_stdin, allow_url, allow_file_uri,   \
@@ -314,8 +248,10 @@ typedef struct gzconn {
     allow_textConnection, allow_rawConnection, allow_sockconn, \
     allow_servsockconn, allow_customConnection,                \
     ignore_blank_string, ignore_clipboard, ignore_stdin,       \
-    ignore_url, ignore_file_uri)                               \
+    ignore_url, ignore_file_uri, source)                       \
 do {                                                           \
+    int nprotect = 0;                                          \
+    PROTECT(documentcontext = DocumentContext()); nprotect++;  \
     if (TYPEOF(ofile) == STRSXP) {                             \
         if (LENGTH(ofile) != 1)                                \
             errorcall(call, "'%s' must be a character string", EncodeChar(PRINTNAME(sym)));\
@@ -340,58 +276,59 @@ do {                                                           \
             else {                                             \
                 url = translateCharUTF8(file);                 \
                 file = mkCharCE(url, CE_UTF8);                 \
+                PROTECT(file); nprotect++;                     \
             }                                                  \
         }                                                      \
         else url = CHAR(file);                                 \
         if (!ignore_blank_string && !(LENGTH(file) > 0)) {     \
             if (allow_blank_string) {                          \
-                assign_null(frame);                            \
+                assign_null(documentcontext);                  \
                 if (assign_returnvalue)                        \
                     returnvalue = PROTECT(ofile);              \
             }                                                  \
             else                                               \
                 errorcall(call, "invalid '%s', must not be \"\"", EncodeChar(PRINTNAME(sym)));\
         }                                                      \
-        else if (!ignore_clipboard && isclipboard(url)) {      \
+        else if (!ignore_clipboard && is_clipboard(url)) {     \
             if (allow_clipboard) {                             \
-                assign_null(frame);                            \
+                assign_null(documentcontext);                  \
                 if (assign_returnvalue)                        \
                     returnvalue = PROTECT(ofile);              \
             }                                                  \
             else                                               \
-                errorcall(call, "invalid '%s', %s", EncodeChar(PRINTNAME(sym)), mustnotbeclipboardmessage);\
+                errorcall(call, "invalid '%s', %s", EncodeChar(PRINTNAME(sym)), must_not_be_clipboard_message);\
         }                                                      \
         else if (!ignore_stdin && streql(url, "stdin")) {      \
             if (allow_stdin) {                                 \
-                assign_null(frame);                            \
+                assign_null(documentcontext);                  \
                 if (assign_returnvalue)                        \
                     returnvalue = PROTECT(ofile);              \
             }                                                  \
             else                                               \
                 errorcall(call, "invalid '%s', must not be \"stdin\"", EncodeChar(PRINTNAME(sym)));\
         }                                                      \
-        else if (!ignore_url && isurl(url)) {                  \
+        else if (!ignore_url && is_url(url)) {                 \
             if (allow_url) {                                   \
-                assign_url(ofile, file, frame);                \
+                assign_url(ofile, file, documentcontext);      \
                 if (assign_returnvalue)                        \
                     returnvalue = PROTECT(ofile);              \
                 if (ofilearg != NULL)                          \
-                    overwrite_ofile(ofilearg, frame);          \
+                    overwrite_ofile(ofilearg, documentcontext);\
             }                                                  \
             else                                               \
                 errorcall(call, "invalid '%s', cannot be a URL", EncodeChar(PRINTNAME(sym)));\
         }                                                      \
-        else if (!ignore_file_uri && strncmp(url, "file://", 7) == 0) {\
+        else if (!ignore_file_uri && is_file_uri(url)) {       \
             if (allow_file_uri) {                              \
-                assign_file_uri(ofile, file, frame, check_not_directory);\
+                assign_file_uri(ofile, file, documentcontext, check_not_directory);\
                 if (assign_returnvalue) {                      \
                     returnvalue = PROTECT(shallow_duplicate(ofile));\
-                    SET_STRING_ELT(returnvalue, 0, STRING_ELT(getInFrame(thispathfileSymbol, frame, FALSE), 0));\
+                    SET_STRING_ELT(returnvalue, 0, STRING_ELT(getInFrame(fileSymbol, documentcontext, FALSE), 0));\
                 }                                              \
                 else if (forcepromise)                         \
-                    getInFrame(thispathfileSymbol, frame, FALSE);\
+                    getInFrame(fileSymbol, documentcontext, FALSE);\
                 if (ofilearg != NULL)                          \
-                    overwrite_ofile(ofilearg, frame);          \
+                    overwrite_ofile(ofilearg, documentcontext);\
             }                                                  \
             else                                               \
                 errorcall(call, "invalid '%s', cannot be a file URI", EncodeChar(PRINTNAME(sym)));\
@@ -400,19 +337,19 @@ do {                                                           \
             if (maybe_chdir) {                                 \
                 SEXP owd = getowd;                             \
                 if (hasowd)                                    \
-                    assign_chdir(ofile, owd, frame);           \
+                    assign_chdir(ofile, owd, documentcontext); \
                 else                                           \
-                    assign_default(ofile, frame, check_not_directory);\
+                    assign_default(ofile, documentcontext, check_not_directory);\
             }                                                  \
-            else assign_default(ofile, frame, check_not_directory);\
+            else assign_default(ofile, documentcontext, check_not_directory);\
             if (assign_returnvalue) {                          \
                 returnvalue = PROTECT(shallow_duplicate(ofile));\
-                SET_STRING_ELT(returnvalue, 0, STRING_ELT(getInFrame(thispathfileSymbol, frame, FALSE), 0));\
+                SET_STRING_ELT(returnvalue, 0, STRING_ELT(getInFrame(fileSymbol, documentcontext, FALSE), 0));\
             }                                                  \
             else if (forcepromise)                             \
-                getInFrame(thispathfileSymbol, frame, FALSE);  \
+                getInFrame(fileSymbol, documentcontext, FALSE);\
             if (ofilearg != NULL)                              \
-                overwrite_ofile(ofilearg, frame);              \
+                overwrite_ofile(ofilearg, documentcontext);    \
         }                                                      \
     }                                                          \
     else {                                                     \
@@ -424,7 +361,7 @@ do {                                                           \
             if (ofilearg != NULL) {                            \
                 if (!identical(ofile, ofilearg)) {             \
                     errorcall(call, "invalid '%s', must be identical to '%s'",\
-                                    "ofile", EncodeChar(PRINTNAME(sym)));\
+                        "ofile", EncodeChar(PRINTNAME(sym)));  \
                 }                                              \
             }                                                  \
             get_description_and_class;                         \
@@ -434,180 +371,122 @@ do {                                                           \
                 streql(klass, "xzfile") ||                     \
                 streql(klass, "fifo"  ))                       \
             {                                                  \
-                assign_file_uri2(description, frame, check_not_directory);\
-                if (forcepromise) getInFrame(thispathfileSymbol, frame, FALSE);\
+                assign_file_uri2(description, documentcontext, check_not_directory);\
+                if (forcepromise) getInFrame(fileSymbol, documentcontext, FALSE);\
             }                                                  \
             else if (streql(klass, "url-libcurl") ||           \
-                     streql(klass, "url-wininet"))             \
+                streql(klass, "url-wininet"))                  \
             {                                                  \
                 if (allow_url) {                               \
-                    assign_url(ScalarString(description), description, frame);\
-                    if (forcepromise) getInFrame(thispathfileSymbol, frame, FALSE);\
+                    assign_url(ScalarString(description), description, documentcontext);\
+                    if (forcepromise) getInFrame(fileSymbol, documentcontext, FALSE);\
                 }                                              \
                 else                                           \
                     errorcall(call, "invalid '%s', cannot be a URL connection", EncodeChar(PRINTNAME(sym)));\
             }                                                  \
             else if (streql(klass, "unz")) {                   \
                 /* we save this error to throw later because   \
-                   it does not indicate an error with the      \
-                   user's usage of the source-like function,   \
-                   but rather an error that the executing      \
-                   document has no path                        \
+                 it does not indicate an error with the        \
+                 user's usage of the source-like function,     \
+                 but rather an error that the executing        \
+                 document has no path                          \
                                                                \
-                   we also assign thispathformsg as the object \
-                   to return for sys.path(for.msg = TRUE)      \
+                 we also assign for_msg as the object to       \
+                 return for this.path(for.msg = TRUE)          \
                                                                \
-                   we also assign thispathassocwfile so that   \
-                   we know this source-call is associated with \
-                   a file, even though that file has no path   \
-                   (well, it has a path, but it cannot be      \
-                    represented by a single string)            \
+                 we also assign associated_with_file so that   \
+                 we know this source-call is associated with   \
+                 a file, even though that file has no path     \
+                 (well, it has a path, but it cannot be        \
+                 represented by a single string)               \
                  */                                            \
                 if (allow_unz) {                               \
-                    SEXP tmp = thisPathInZipFileError(R_NilValue, description);\
-                    INCREMENT_NAMED_defineVar(thispatherrorSymbol, tmp, frame);\
-                    R_LockBinding(thispatherrorSymbol, frame); \
-                    tmp = ScalarString(description);           \
-                    INCREMENT_NAMED_defineVar(thispathformsgSymbol, tmp, frame);\
-                    R_LockBinding(thispathformsgSymbol, frame);\
-                    defineVar(thispathassocwfileSymbol, R_NilValue, frame);\
-                    R_LockBinding(thispathassocwfileSymbol, frame);\
-                    assign_done(frame);                        \
+                    INCREMENT_NAMED_defineVar(errcndSymbol              , thisPathInZipFileError(R_NilValue, description), documentcontext);\
+                    INCREMENT_NAMED_defineVar(for_msgSymbol             , ScalarString(description)                      , documentcontext);\
+                                    defineVar(associated_with_fileSymbol, R_TrueValue                                    , documentcontext);\
                 }                                              \
                 else                                           \
                     errorcall(call, "invalid '%s', cannot be a unz connection", EncodeChar(PRINTNAME(sym)));\
             }                                                  \
-            else if (isclipboard(klass)) {                     \
+            else if (is_clipboard(klass)) {                    \
                 if (allow_clipboard)                           \
-                    assign_null(frame);                        \
+                    assign_null(documentcontext);              \
                 else                                           \
                     errorcall(call, "invalid '%s', cannot be a clipboard connection", EncodeChar(PRINTNAME(sym)));\
             }                                                  \
             else if (streql(klass, "pipe")) {                  \
                 if (allow_pipe)                                \
-                    assign_null(frame);                        \
+                    assign_null(documentcontext);              \
                 else                                           \
                     errorcall(call, "invalid '%s', cannot be a pipe connection", EncodeChar(PRINTNAME(sym)));\
             }                                                  \
             else if (streql(klass, "terminal")) {              \
                 if (allow_terminal)                            \
-                    assign_null(frame);                        \
+                    assign_null(documentcontext);              \
                 else                                           \
                     errorcall(call, "invalid '%s', cannot be a terminal connection", EncodeChar(PRINTNAME(sym)));\
             }                                                  \
             else if (streql(klass, "textConnection")) {        \
                 if (allow_textConnection)                      \
-                    assign_null(frame);                        \
+                    assign_null(documentcontext);              \
                 else                                           \
                     errorcall(call, "invalid '%s', cannot be a textConnection", EncodeChar(PRINTNAME(sym)));\
             }                                                  \
             else if (streql(klass, "rawConnection")) {         \
                 if (allow_rawConnection)                       \
-                    assign_null(frame);                        \
+                    assign_null(documentcontext);              \
                 else                                           \
                     errorcall(call, "invalid '%s', cannot be a rawConnection", EncodeChar(PRINTNAME(sym)));\
             }                                                  \
             else if (streql(klass, "sockconn")) {              \
                 if (allow_sockconn)                            \
-                    assign_null(frame);                        \
+                    assign_null(documentcontext);              \
                 else                                           \
                     errorcall(call, "invalid '%s', cannot be a sockconn", EncodeChar(PRINTNAME(sym)));\
             }                                                  \
             else if (streql(klass, "servsockconn")) {          \
                 if (allow_servsockconn)                        \
-                    assign_null(frame);                        \
+                    assign_null(documentcontext);              \
                 else                                           \
                     errorcall(call, "invalid '%s', cannot be a servsockconn", EncodeChar(PRINTNAME(sym)));\
             }                                                  \
             else {                                             \
                 if (allow_customConnection) {                  \
                     /* same as "unz", we save the error and    \
-                       the description for                     \
-                       sys.path(for.msg = TRUE)                \
+                     the description for                       \
+                     sys.path(for.msg = TRUE)                  \
                                                                \
-                       however, we do not save                 \
-                       thispathassocwfile because we don't     \
-                       know if this connection has an          \
-                       associated file                         \
+                     however, we do not save                   \
+                     associated_with_file because we don't     \
+                     know if this connection has an            \
+                     associated file                           \
                      */                                        \
-                    SEXP tmp = thisPathUnrecognizedConnectionClassError(R_NilValue, Rcon_or_summary(Rcon, summary));\
-                    INCREMENT_NAMED_defineVar(thispatherrorSymbol, tmp, frame);\
-                    R_LockBinding(thispatherrorSymbol, frame); \
-                    tmp = ScalarString(description);           \
-                    INCREMENT_NAMED_defineVar(thispathformsgSymbol, tmp, frame);\
-                    R_LockBinding(thispathformsgSymbol, frame);\
-                    assign_done(frame);                        \
+                    INCREMENT_NAMED_defineVar(errcndSymbol , thisPathUnrecognizedConnectionClassError(R_NilValue, Rcon_or_summary), documentcontext);\
+                    INCREMENT_NAMED_defineVar(for_msgSymbol, ScalarString(description)                                            , documentcontext);\
                 }                                              \
                 else                                           \
                     errorcall(call, "invalid '%s', cannot be a connection of class '%s'",\
-                              EncodeChar(PRINTNAME(sym)), EncodeChar(mkChar(klass)));\
+                        EncodeChar(PRINTNAME(sym)), EncodeChar(mkChar(klass)));\
             }                                                  \
         }                                                      \
         if (assign_returnvalue) returnvalue = PROTECT(ofile);  \
     }                                                          \
+    INCREMENT_NAMED_defineVar(sourceSymbol, (source), documentcontext);\
+    if (as_binding) {                                          \
+        INCREMENT_NAMED_defineVar(documentcontextSymbol, documentcontext, frame);\
+        R_LockBinding(documentcontextSymbol, frame);           \
+    } else {                                                   \
+        setAttrib(frame, documentcontextSymbol, documentcontext);\
+    }                                                          \
+    UNPROTECT(nprotect);                                       \
     set_R_Visible(TRUE);                                       \
 } while (0)
 
 
-extern SEXP sys_call(SEXP which, SEXP rho);
-#define getCurrentCall(rho) eval(expr_sys_call, (rho))
-
-
-/* doesn't work in general, for example sys.function() duplicates its return value */
-// #define identical(x, y) ((x) == (y))
-
-
-/* this is the default implementation of identical() */
-/* num.eq = TRUE                 num_as_bits = FALSE      0
-   single.NA = TRUE              NA_as_bits = FALSE       0
-   attrib.as.set = TRUE          attr_by_order = FALSE    0
-   ignore.bytecode = TRUE        use_bytecode = FALSE     0
-   ignore.environment = FALSE    use_cloenv = TRUE        16
-   ignore.srcref = TRUE          use_srcref = FALSE       0
-   extptr.as.ref = FALSE         extptr_as_ref = FALSE    0
- */
-// #define identical(x, y) R_compute_identical((x), (y), 16)
-
-
-/* num.eq = FALSE                num_as_bits = TRUE       1
-   single.NA = FALSE             NA_as_bits = TRUE        2
-   attrib.as.set = FALSE         attr_by_order = TRUE     4
-   ignore.bytecode = FALSE       use_bytecode = TRUE      8
-   ignore.environment = FALSE    use_cloenv = TRUE        16
-   ignore.srcref = FALSE         use_srcref = TRUE        32
-   extptr.as.ref = TRUE          extptr_as_ref = TRUE     64
- */
-#define identical(x, y) R_compute_identical((x), (y), 127)
-
-
-extern int gui_rstudio;
-extern Rboolean has_tools_rstudio;
-extern Rboolean init_tools_rstudio(Rboolean skipCheck);
-
-
-#define in_rstudio                                             \
-    ((gui_rstudio != -1) ? (gui_rstudio) : (gui_rstudio = asLogical(getFromMyNS(_gui_rstudioSymbol))))
-
-
-extern int maybe_unembedded_shell;
-
-
-#define is_maybe_unembedded_shell                              \
-    ((maybe_unembedded_shell != -1) ? (maybe_unembedded_shell) : (maybe_unembedded_shell = asLogical(getFromMyNS(_maybe_unembedded_shellSymbol))))
-
-
-#define get_debugSource                                        \
-    ((has_tools_rstudio) ? getFromMyNS(_debugSourceSymbol) : R_UnboundValue)
-
-
 #define wrong_nargs_to_External(nargs, name, expected_nargs)   \
     (((nargs) == 1) ? "%d argument passed to .External(%s) which requires %s" :\
                       "%d arguments passed to .External(%s) which requires %s"),\
                      (nargs), (name), (expected_nargs)
 
 
-extern SEXP get_sys_parents(SEXP rho);
-extern int get_sys_parent(int n, SEXP rho);
-
-
-#endif /* #ifndef THISPATHDEFN_H */
+#endif /* #ifndef R_THISPATH_THISPATHDEFN_H */
