@@ -1,5 +1,5 @@
-#ifndef TRANSLATIONS_H
-#define TRANSLATIONS_H
+#ifndef R_THISPATH_TRANSLATIONS_H
+#define R_THISPATH_TRANSLATIONS_H
 
 
 #ifdef ENABLE_NLS
